import asyncio
import logging
import sys

from aiogram import Bot, Dispatcher, F
from aiogram.client.default import DefaultBotProperties
from aiogram.enums import ParseMode
from aiogram.filters import CommandStart, Command, StateFilter
from aiogram.types import Message, BufferedInputFile, CallbackQuery
from aiogram.fsm.context import FSMContext
from aiogram.fsm.state import StatesGroup, State

import chess

from my_credentials import API_TOKEN
from chess_utils import generate_board_image, first_tap_keyboard, second_tap_keyboard, make_move, make_bot_move


dp = Dispatcher()
boards = {}

class ChessMatchStatesGroup(StatesGroup):
    has_chosen_piece = State()


@dp.message(CommandStart())
async def send_welcome(message: Message) -> None:
    await message.reply("Привет! Я ваш первый Telegram-бот.")

@dp.message(Command("newgame"))
async def start_game(message: Message) -> None:
    initial_board = chess.Board()
    boards[message.from_user.id] = initial_board
    await message.answer_photo(
        photo=BufferedInputFile(generate_board_image(initial_board), "game.png"),
        caption="Игра началась! Делай первый ход. Нажми на клавиатуру: первым нажатием выбери фигуру, вторым - куда она пойдет",
        reply_markup=first_tap_keyboard(initial_board),
    )

@dp.callback_query(StateFilter(None), F.data.startswith("turn_"))
async def process_from_move(message: CallbackQuery, state: FSMContext) -> None:
    if message.from_user.id not in boards:
        return
    board = boards[message.from_user.id]
    square_name = message.data.removeprefix("turn_")
    await state.set_data({"square": square_name})
    await state.set_state(ChessMatchStatesGroup.has_chosen_piece)
    await message.message.edit_reply_markup(
        reply_markup=second_tap_keyboard(board, square_name),
    )


@dp.callback_query(ChessMatchStatesGroup.has_chosen_piece, F.data == "turn2_cancel")
async def process_to_cancel(message: Message, state: FSMContext) -> None:
    if message.from_user.id not in boards:
        return
    board = boards[message.from_user.id]
    await state.clear()
    await message.message.edit_reply_markup(
        reply_markup=first_tap_keyboard(board),
    )


@dp.callback_query(ChessMatchStatesGroup.has_chosen_piece, F.data.startswith("turn2_"))
async def process_to_move(message: Message, state: FSMContext) -> None:
    async def send_end_of_game():
        await message.message.delete()
        await message.message.answer_photo(
            photo=BufferedInputFile(generate_board_image(board), "game.png"),
            caption=f"Игра закончена! Результат - {outcome.result()}. Новая игра - /newgame"
        )
        del boards[message.from_user.id]
        await state.clear()

    if message.from_user.id not in boards:
        return

    board = boards[message.from_user.id]
    square_name = message.data.removeprefix("turn2_")
    data = await state.get_data()

    outcome = make_move(board, f"{data.get('square')}{square_name}")
    if outcome is not None:
        await send_end_of_game()
        return

    outcome = make_bot_move(board)
    if outcome is not None:
        await send_end_of_game()
        return

    await message.message.delete()
    await message.message.answer_photo(
        photo=BufferedInputFile(generate_board_image(board), "game.png"),
        caption=f"Делайте следующий ход!",
        reply_markup=first_tap_keyboard(board),
    )
    await state.clear()


async def main() -> None:
    bot = Bot(token=API_TOKEN, default=DefaultBotProperties(parse_mode=ParseMode.HTML))
    await dp.start_polling(bot)


if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO, stream=sys.stdout)
    asyncio.run(main())
